import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import java.lang.Integer;
import java.util.InputMismatchException;
import java.io.BufferedReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.Writer;
import java.util.AbstractCollection;
import java.util.Arrays;
import java.util.StringTokenizer;
import java.util.LinkedList;
import java.util.Random;
import java.io.InputStream;

/**
 * Built using CHelper plug-in
 * Actual solution is at the top
 * @author Andrey Rechitsky (arechitsky@gmail.com)
 */
public class Main {
    public static void main(String[] args) {
        OutputStream generatorOutputStream;
        try {
            generatorOutputStream = new FileOutputStream("BRIDGES.IN");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        FastPrinter generatorOut = new FastPrinter(generatorOutputStream);
        int count = 1000;
        long seed = 123123;
        if (args.length >= 1) count = Integer.parseInt(args[0]);
        if (args.length >= 2) seed = Long.parseLong(args[1]);
        InputGenerator generator = new InputGenerator();
        generator.generate(count, seed, generatorOut);
        generatorOut.close();

        InputStream solverInputStream;
        try {
            solverInputStream = new FileInputStream("BRIDGES.IN");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        OutputStream solverOutputStream;
        try {
            solverOutputStream = new FileOutputStream("BRIDGES.OUT");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        FastScanner in = new FastScanner(solverInputStream);
        FastPrinter out = new FastPrinter(solverOutputStream);
        Bridges solver = new Bridges();
        int testCount = Integer.parseInt(in.next());
        for (int i = 1; i <= testCount; i++)
            solver.solve(i, in, out);
        out.close();
    }
}

class InputGenerator {
    Random rnd;

    public void generate(int count, long seed, FastPrinter out){
        rnd = new Random(seed);

        out.printLine(count);

        int fixedCount = 10;
        printTest(out, simpleRect(1, 2));
        printTest(out, bridgeLocked());
        printTest(out, bridgeLoop());
        printTest(out, spiral(6));
        printTest(out, spiral(7));
        printTest(out, snake(3, 3));
        printTest(out, snake(7, 7));
        printTest(out, snake(5, 8));
        printTest(out, spiral(100));
        printTest(out, snake(100, 100));

        for (int i = fixedCount; i < count; i++) {
            int n = rnd.nextInt(100)+1;
            int m = rnd.nextInt(100)+1;
            if (n == 1 && m == 1) m++;
            double wallRate = 0.1 + 0.4 * rnd.nextDouble();
            double bridgeRate = 0.1 + 0.4 * rnd.nextDouble();
            printTest(out, random(n, m, wallRate, bridgeRate));
        }
    }

    char[][] simpleRect(int n, int m){
        char[][] field = new char[n][m];
        for (int i = 0; i < n; i++) {
            Arrays.fill(field[i], '.');
        }
        field[0][0] = 'S';
        field[n - 1][m - 1] = 'E';
        return field;
    }

    char[][] bridgeLocked(){
        char[][] field = new char[2][];
        field[0] = "SB.".toCharArray();
        field[1] = "#E#".toCharArray();
        return field;
    }

    char[][] bridgeLoop(){
        char[][] field = new char[3][];
        field[0] = "#E#".toCharArray();
        field[1] = "SB.".toCharArray();
        field[2] = "#..".toCharArray();
        return field;
    }

    /**
     * SBBBBB.
     * B.BBB.B
     * BB.B.BB
     * BBBEBBB
     * BBB..BB
     * BB.BB.B
     * B.BBBB.
     *
     * SBBBB.
     * B.BB.B
     * BB..BB
     * BBBEBB
     * BB.B.B
     * B.BBB.
     */
    char[][] spiral(int side){
        char[][] field = new char[side][side];
        for (int i = 0; i < side; i++) {
            Arrays.fill(field[i], 'B');
        }
        int half = side / 2;
        int other_half = side - half - 1;
        for (int i = 0; i < half; i++) {
            field[i][i] = '.';
            field[i][side - 1 - i] = '.';
        }
        for (int i = 0; i < other_half; i++) {
            field[side - 1 - i][i + 1] = '.';
            field[side - 1 - i][side - 1 - i] = '.';
        }
        field[0][0] = 'S';
        field[half][half] = 'E';
        return field;
    }

    /**
     *
     * #E#.B.#
     * SB.B.B.
     * #.#.#.B
     * .B.B.B.
     * B.#.#.#
     * .B.B.B.
     * #.B.#..
     */
    char[][] snake(int n, int m){
        char[][] field = new char[n][m];
        for (int i = 0; i < n; i++) {
            Arrays.fill(field[i], '#');
        }
        // can generate only for 4*k+3 x 4*l+3
        int k = (n - 3) / 4;
        int l = (m - 3) / 4;

        field[0][0] = '#';
        field[0][1] = 'E';
        for (int i = 0; i < l; i++) {
            field[0][2 + i * 4] = '#';
            field[0][2 + i * 4 + 1] = field[0][2 + i * 4 + 3] = '.';
            field[0][2 + i * 4 + 2] = 'B';
        }
        field[0][2 + l * 4] = '#';

        field[1][0] = 'S';
        field[1][1] = 'B';
        for (int i = 0; i < l; i++) {
            field[1][2 + i * 4] = field[1][2 + i * 4 + 2] = '.';
            field[1][2 + i * 4 + 1] = field[1][2 + i * 4 + 3] = 'B';
        }
        field[1][2 + l * 4] = '.';

        for (int i = 0; i < k; i++) {
            for (int o = 0; o < 4; o++) {
                for (int p = 0; p < 2; p++) {
                    if ((o + p) % 2 == 1) field[2 + i * 4 + o][p] = '.';
                    else field[2 + i * 4 + o][p] = 'B';
                }
            }
            field[2 + i * 4][0] = '#';
            for (int j = 0; j < l; j++) {
                for (int o = 0; o < 4; o++) {
                    for (int p = 0; p < 4; p++) {
                        if ((o + p) % 2 == 1) field[2 + i * 4 + o][2 + j * 4 + p] = '.';
                        else if (o % 2 == 1) field[2 + i * 4 + o][2 + j * 4 + p] = 'B';
                        else field[2 + i * 4 + o][2 + j * 4 + p] = '#';
                    }
                }
            }
            field[2 + i * 4][2 + l * 4] = 'B';
            field[2 + i * 4 + 1][2 + l * 4] = field[2 + i * 4 + 3][2 + l * 4] = '.';
            field[2 + i * 4 + 2][2 + l * 4] = '#';
        }

        field[2 + k * 4][0] = '#';
        field[2 + k * 4][1] = '.';
        for (int i = 0; i < l; i++) {
            field[2 + k * 4][2 + i * 4] = 'B';
            field[2 + k * 4][2 + i * 4 + 1] = field[2 + k * 4][2 + i * 4 + 3] = '.';
            field[2 + k * 4][2 + i * 4 + 2] = '#';
        }
        field[2 + k * 4][2 + l * 4] = '.';
        return field;
    }

    char[][] random(int n, int m, double wallRate, double bridgeRate){
        char[][] field = new char[n][m];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                if (rnd.nextDouble() < wallRate) field[i][j] = '#';
                else if (rnd.nextDouble() < bridgeRate) field[i][j] = 'B';
                else field[i][j] = '.';
            }
        }
        int sr = rnd.nextInt(n);
        int sc = rnd.nextInt(m);
        int er;
        int ec;
        do {
            er = rnd.nextInt(n);
            ec = rnd.nextInt(m);
        } while (er == sr && ec == sc);
        field[sr][sc] = 'S';
        field[er][ec] = 'E';
        return field;
    }

    void printTest(FastPrinter out, char[][] field){
        out.printLine("%d %d", field.length, field[0].length);
        for (int i = 0; i < field.length; i++) {
            for (int j = 0; j < field[i].length; j++) {
                out.print(field[i][j]);
            }
            out.printLine();
        }
    }
}

class Bridges {
    char[][] field;
    boolean[][][] visited;
    int[][][] dist;
    LinkedList<Position> queue;

    public void solve(int testNumber, FastScanner in, FastPrinter out) {
        int n = in.nextInt();
        int m = in.nextInt();
        field = new char[n][];
        for (int i = 0; i < n; i++) {
            field[i] = in.next().toCharArray();
        }

        // Let's split each cell of our labyrinth on two layers:
        // from layer 0 we can go only up or down, and from layer 1 - left or right.
        // So, if we going from some layer of one cell to free cell, we can move to any layer of that cell.
        // But if we going from some layer of one cell to bridge cell, we can move to only the same layer.

        // BFS in splitted labyrinth
        visited = new boolean[n][m][2];
        dist = new int[n][m][2];
        queue = new LinkedList<Position>();
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                if (field[i][j] != 'S') continue;
                queue.add(new Position(i, j, 0));
                visited[i][j][0] = true;
                dist[i][j][0] = 0;
                queue.add(new Position(i, j, 1));
                visited[i][j][1] = true;
                dist[i][j][1] = 0;
            }
        }
        while (!queue.isEmpty()) {
            Position cur = queue.poll();
            if (cur.layer == 0) { //up-down
                if (cur.row > 0) { //going up
                    process(cur, new Position(cur.row - 1, cur.column, 0));
                    process(cur, new Position(cur.row - 1, cur.column, 1));
                }
                if (cur.row < n - 1) { //going down
                    process(cur, new Position(cur.row + 1, cur.column, 0));
                    process(cur, new Position(cur.row + 1, cur.column, 1));
                }
            } else //left-right
            {
                if (cur.column > 0) { //going left
                    process(cur, new Position(cur.row, cur.column - 1, 0));
                    process(cur, new Position(cur.row, cur.column - 1, 1));
                }
                if (cur.column < m - 1) { //going right
                    process(cur, new Position(cur.row, cur.column + 1, 0));
                    process(cur, new Position(cur.row, cur.column + 1, 1));
                }
            }
        }
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                if (field[i][j] != 'E') continue;
                // It's guaranteed that visited and dist for any free cell are the same for both layers
                if (!visited[i][j][0]) {
                    out.printLine(-1);
                    return;
                } else {
                    out.printLine(dist[i][j][0]);
                    return;
                }
            }
        }
    }

    /**
     * Checks, if we can move from Position 'from' to Position 'to'.
     * Adds Position 'to' to BFS queue if we can.
     */
    void process(Position from, Position to) {
        if (visited[to.row][to.column][to.layer]) return; // won't move on visited
        if (field[to.row][to.column] == '#') return; //can't move to wall
        if (from.layer != to.layer && field[to.row][to.column] == 'B') return; // can't move to another layer of bridge
        queue.add(to);
        visited[to.row][to.column][to.layer] = true;
        dist[to.row][to.column][to.layer] = dist[from.row][from.column][from.layer] + 1;
    }

    class Position {
        /**
         * Row and column
         */
        public int row, column;

        /**
         * Layer
         * 1 on bridge, 0 under bridge
         */
        public int layer;

        public Position(int row, int column, int layer) {
            this.row = row;
            this.column = column;
            this.layer = layer;

        }
    }
}

class FastScanner {

    private BufferedReader reader;
    private StringTokenizer st;

    public FastScanner(InputStream stream) {
        this.reader = new BufferedReader(new InputStreamReader(stream));
        this.st = new StringTokenizer("");
    }

    public int nextInt() {
        return Integer.parseInt(next());
    }

    public String next() {
        while (!st.hasMoreTokens()) {
            st = new StringTokenizer(readLine());
        }
        return st.nextToken();
    }

    private String readLine() {
        try {
            String line = reader.readLine();
            if (line == null) throw new InputMismatchException();
            return line;
        } catch (IOException e) {
            throw new InputMismatchException();
        }
    }

}

class FastPrinter {
    private final PrintWriter writer;

    public FastPrinter(OutputStream outputStream) {
        writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(outputStream)));
    }

    public void close() {
        writer.close();
    }

    public void printLine(long i) {
        writer.println(i);
    }

    public void printLine(String s) {
        writer.println(s);
    }

    public void printLine(String format, Object... args) {
        printLine(String.format(format, args));
    }

    public void printLine() {
        writer.println();
    }

    public void print(char c) {
        writer.print(c);
    }
}

