import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import java.util.InputMismatchException;
import java.io.BufferedReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.FileInputStream;
import java.io.Writer;
import java.util.AbstractCollection;
import java.util.StringTokenizer;
import java.util.LinkedList;
import java.io.InputStream;

/**
 * Built using CHelper plug-in
 * Actual solution is at the top
 * @author Andrey Rechitsky (arechitsky@gmail.com)
 */
public class Bridges_ar {
    public static void main(String[] args) {
        InputStream inputStream;
        try {
            inputStream = new FileInputStream("BRIDGES.IN");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        OutputStream outputStream = System.out;
        FastScanner in = new FastScanner(inputStream);
        FastPrinter out = new FastPrinter(outputStream);
        Bridges solver = new Bridges();
        int testCount = Integer.parseInt(in.next());
        for (int i = 1; i <= testCount; i++)
            solver.solve(i, in, out);
        out.close();
    }
}

class Bridges {
    char[][] field;
    boolean[][][] visited;
    int[][][] dist;
    LinkedList<Position> queue;

    public void solve(int testNumber, FastScanner in, FastPrinter out) {
        int n = in.nextInt();
        int m = in.nextInt();
        field = new char[n][];
        for (int i = 0; i < n; i++) {
            field[i] = in.next().toCharArray();
        }

        // Let's split each cell of our labyrinth on two layers:
        // from layer 0 we can go only up or down, and from layer 1 - left or right.
        // So, if we going from some layer of one cell to free cell, we can move to any layer of that cell.
        // But if we going from some layer of one cell to bridge cell, we can move to only the same layer.

        // BFS in splitted labyrinth
        visited = new boolean[n][m][2];
        dist = new int[n][m][2];
        queue = new LinkedList<Position>();
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                if (field[i][j] != 'S') continue;
                queue.add(new Position(i, j, 0));
                visited[i][j][0] = true;
                dist[i][j][0] = 0;
                queue.add(new Position(i, j, 1));
                visited[i][j][1] = true;
                dist[i][j][1] = 0;
            }
        }
        while (!queue.isEmpty()) {
            Position cur = queue.poll();
            if (cur.layer == 0) { //up-down
                if (cur.row > 0) { //going up
                    process(cur, new Position(cur.row - 1, cur.column, 0));
                    process(cur, new Position(cur.row - 1, cur.column, 1));
                }
                if (cur.row < n - 1) { //going down
                    process(cur, new Position(cur.row + 1, cur.column, 0));
                    process(cur, new Position(cur.row + 1, cur.column, 1));
                }
            } else //left-right
            {
                if (cur.column > 0) { //going left
                    process(cur, new Position(cur.row, cur.column - 1, 0));
                    process(cur, new Position(cur.row, cur.column - 1, 1));
                }
                if (cur.column < m - 1) { //going right
                    process(cur, new Position(cur.row, cur.column + 1, 0));
                    process(cur, new Position(cur.row, cur.column + 1, 1));
                }
            }
        }
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                if (field[i][j] != 'E') continue;
                // It's guaranteed that visited and dist for any free cell are the same for both layers
                if (!visited[i][j][0]) {
                    out.printLine(-1);
                    return;
                } else {
                    out.printLine(dist[i][j][0]);
                    return;
                }
            }
        }
    }

    /**
     * Checks, if we can move from Position 'from' to Position 'to'.
     * Adds Position 'to' to BFS queue if we can.
     */
    void process(Position from, Position to) {
        if (visited[to.row][to.column][to.layer]) return; // won't move on visited
        if (field[to.row][to.column] == '#') return; //can't move to wall
        if (from.layer != to.layer && field[to.row][to.column] == 'B') return; // can't move to another layer of bridge
        queue.add(to);
        visited[to.row][to.column][to.layer] = true;
        dist[to.row][to.column][to.layer] = dist[from.row][from.column][from.layer] + 1;
    }

    class Position {
        /**
         * Row and column
         */
        public int row, column;

        /**
         * Layer
         * 1 on bridge, 0 under bridge
         */
        public int layer;

        public Position(int row, int column, int layer) {
            this.row = row;
            this.column = column;
            this.layer = layer;

        }
    }
}

class FastScanner {

    private BufferedReader reader;
    private StringTokenizer st;

    public FastScanner(InputStream stream) {
        this.reader = new BufferedReader(new InputStreamReader(stream));
        this.st = new StringTokenizer("");
    }

    public int nextInt() {
        return Integer.parseInt(next());
    }

    public String next() {
        while (!st.hasMoreTokens()) {
            st = new StringTokenizer(readLine());
        }
        return st.nextToken();
    }

    private String readLine() {
        try {
            String line = reader.readLine();
            if (line == null) throw new InputMismatchException();
            return line;
        } catch (IOException e) {
            throw new InputMismatchException();
        }
    }

}

class FastPrinter {
    private final PrintWriter writer;

    public FastPrinter(OutputStream outputStream) {
        writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(outputStream)));
    }

    public void close() {
        writer.close();
    }

    public void printLine(long i) {
        writer.println(i);
    }

}

