﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace DiscountTests
{
    class DiscountTests
    {
        static void Main(string[] args)
        {
            if (args.Length == 1)
                (new Tests(int.Parse(args[0]))).Run();
            else
            {
                (new Tests()).Run();
            }
        }

        public class Tests
        {
            private struct Data
            {
                public readonly int A;
                public readonly int B;
                public readonly int C;
                public const int Min = 1;
                public const int Max = 1000000000;

                public Data(int a, int b, int c)
                {
                    A = a;
                    B = b;
                    C = c;
                }

                private bool Equals(Data other)
                {
                    return A == other.A && B == other.B && C == other.C;
                }

                public override bool Equals(object obj)
                {
                    if (ReferenceEquals(null, obj)) return false;
                    return obj is Data && Equals((Data)obj);
                }

                public override int GetHashCode()
                {
                    unchecked
                    {
                        var hashCode = A;
                        hashCode = (hashCode * 397) ^ B;
                        hashCode = (hashCode * 397) ^ C;
                        return hashCode;
                    }
                }
            }

            #region Fields
            private readonly StreamWriter _writer;
            private const string FileName = "DISCOUNT.IN";
            private readonly int _tests = 10000;
            #endregion

            #region Ctor
            public Tests()
            {
                _writer = new StreamWriter(new FileStream(FileName, FileMode.Create));
            }
            public Tests(int tests)
            {
                _writer = new StreamWriter(new FileStream(FileName, FileMode.Create));
                _tests = tests;
            }
            #endregion

            #region Methods

            public void Run()
            {

                var generatedData = GenerateTests();
                _writer.WriteLine(generatedData.Length.ToString());
                PrintTest(generatedData);
                _writer.Close();
            }

            private void PrintTest(Data data)
            {
                _writer.WriteLine("{0} {1} {2}", data.A, data.B, data.C);
            }

            private void PrintTest(IEnumerable<Data> data)
            {
                foreach (var item in data)
                {
                    PrintTest(item);
                }
            }

            private Data[] GenerateTests()
            {
                Random random = new Random(42);
                var defaultData = GetDefaultData();
                var minMaxData = GetMinMaxData();
                var firstData = new Data[defaultData.Length + minMaxData.Length];
                defaultData.CopyTo(firstData, 0);
                minMaxData.CopyTo(firstData, defaultData.Length);
                var generatedData = new HashSet<Data>(firstData);

                while (generatedData.Count < _tests)
                {
                    for (int a = 10; a <= Data.Max; a *= 10)
                    {
                        for (int b = 10; b <= Data.Max; b *= 10)
                        {
                            for (int c = 10; c <= Data.Max; c *= 10)
                            {
                                generatedData.Add(new Data(random.Next(Data.Min, a),
                                                        random.Next(Data.Min, b), 
                                                        random.Next(Data.Min, c)));
                                if (generatedData.Count == _tests)
                                    return generatedData.ToArray();
                            }
                        }
                    }
                }

                return generatedData.ToArray();
            }

            private Data[] GetDefaultData()
            {
                return new[]
                {
                    new Data(8,11,6),
                    new Data(10, 10, 10)
                };
            }

            private Data[] GetMinMaxData()
            {
                int[] minMax = { Data.Min, Data.Max };
                var list = new List<Data>();
                for (int s1 = 0; s1 < minMax.Length; ++s1)
                {
                    for (int s2 = 0; s2 < minMax.Length; ++s2)
                    {
                        for (int s3 = 0; s3 < minMax.Length; ++s3)
                            list.AddRange(minMax.Select(l => new Data(minMax[s1], minMax[s2], minMax[s3])));
                    }
                }
                return list.ToArray();
            }
            #endregion
        }
    }
}
