﻿/*
@author Efim Konstantinov (wertrix.ek@gmail.com)
*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace StudentsTests
{
    class StudentsTests
    {
        static void Main(string[] args)
        {
            if(args.Length==1)
                (new Tests(int.Parse(args[0]))).Run();
            else
            {
                (new Tests()).Run();
            }
        }

        public class Tests
        {
            private class Data
            {
                public readonly List<List<string>> Students;
                public const int MaxStudents = 1000;
                public const int MinStudents = 1;
                public const int MinNameLength = 1;
                public const int MaxNameLength = 10;

                public Data(List<List<string>> students)
                {
                    Students = students;
                }

                public Data()
                {
                    Students = new List<List<string>>();
                }
            }

            #region Fields
            private readonly StreamWriter _writer;
            private const string FileName = "STUDENTS.IN";
            private int _tests = 1000;
            #endregion

            #region Ctor
            public Tests()
            {
                _writer = new StreamWriter(new FileStream(FileName, FileMode.Create));
            }

            public Tests(int tests)
            {
                _writer = new StreamWriter(new FileStream(FileName, FileMode.Create));
                _tests = tests;
            }
            #endregion

            #region Methods

            public void Run()
            {
                var generatedData = GenerateTests();
                _writer.WriteLine(generatedData.Length.ToString());
                PrintTest(generatedData);
                _writer.Close();
            }

            private void PrintTest(Data data)
            {
                _writer.WriteLine(data.Students.Count.ToString());
                foreach (var student in data.Students)
                {
                    _writer.WriteLine(string.Join(" ", student.ToArray()));
                }
            }

            private void PrintTest(IEnumerable<Data> data)
            {
                foreach (var item in data)
                {
                    PrintTest(item);
                }
            }

            private Data[] GenerateTests()
            {
                Random random = new Random(42);
                var defaultData = GetDefaultData();
                var generatedData = new HashSet<Data>(defaultData);

                int tn = _tests - defaultData.Length;
                int[] nnn = { 5, 10, 20, 30, 50, 100, 200, 300, 500, 1000 };
                int t = 0;

                while (t < tn)
                {
                    for (int i = 0; i < nnn.Length; ++i)
                    {
                        var names = new List<string>(nnn[i]/3);
                        for (int n = 0; n < names.Capacity; ++n)
                        {
                            names.Add(GetName(random, random.Next(Data.MinNameLength, Data.MaxNameLength)));
                        }
                        int studentsNumber = random.Next(Data.MinStudents, nnn[i]);
						Console.WriteLine(studentsNumber);
                        var test = new Data();
                        for (int s = 0; s < studentsNumber; ++s)
                        {
                            var student = new List<string>();
                            int fio = random.Next(2, 3);
                            for (int f = 0; f < fio; ++f)
                            {
                                student.Add(f == 1 && t % 5 != 0
                                    ? names[random.Next(0, names.Count)]
                                    : GetName(random, random.Next(Data.MinNameLength, Data.MaxNameLength)));
                            }
                            if (t % 6 == 0)
                            {
                                student.Add("m");
                            }
                            else
                            {
                                student.Add(random.Next(100) > 50 ? "w" : "m");
                            }
                            test.Students.Add(student);
                        }
                        generatedData.Add(test);
                        t++;
                        if(t==tn) return generatedData.ToArray();
                    }
                }
                return generatedData.ToArray();
            }

            private Data[] GetDefaultData()
            {
                return new[]
                {
                    new Data(new List<List<string>>()
                    {
                        new List<string>() { "Petrova", "Anna", "Petrovna", "w" },
                        new List<string>() { "Alieva", "Inna", "w" },
                        new List<string>() { "Ivanova", "Inna", "Ivanovna", "w" },
                        new List<string>() { "Sidorova", "Anna", "Sidorovna", "w" }
                    }),
                    new Data(new List<List<string>>()
                    {
                        new List<string>() { "Ivanov", "Ivan", "Ivanovich", "m" }
                    }),
                    new Data(new List<List<string>>()
                    {
                        new List<string>() { "Petrov", "Petr", "Petrovich", "m" },
                        new List<string>() { "Petrova", "Petra", "Petrovna", "w" }
                    })
                };
            }

            private string GetName(Random random, int length)
            {
                var builder = new StringBuilder();
                builder.Append((char)('A' + random.Next(26)));
                for (int i = 1; i < length; ++i)
                {
                    builder.Append((char)('a' + random.Next(26)));
                }
                return builder.ToString();
            }
            #endregion
        }
    }
}
